package JournalController;

$OBJC_EXPORT{'sheetDidEnd:returnCode:contextInfo:'} = {
    'args' => '@i^v',
    'return' => 'v',
};

$OBJC_EXPORT{'writeUpload:'} = {
    'args' => '@',
    'return' => 'v',
};

$OBJC_EXPORT{'writeOverwrite:'} = {
    'args' => '@',
    'return' => 'v',
};

$OBJC_EXPORT{'numberOfRowsInTableView:'} = {
    'args' => '@',
    'return' => 'i',
};

$OBJC_EXPORT{'tableView:objectValueForTableColumn:row:'} = {
    'args' => '@@i',
    'return' => '@',
};

$OBJC_EXPORT{'clickOfflineEntry:'} = {
    'args' => '@',
    'return' => 'v',
};

use Foundation;
use Foundation::Functions;
use AppKit;
use AppKit::Functions;

use HTTP::Cookies;
use SOAP::Lite;
use Digest::MD5 'md5_hex';

use JournalEntry;

@ISA = qw(Exporter);

sub new {
    # Typical Perl constructor
    # See 'perltoot' for details
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = {};

    # Outlets
    $self->{'Window'}          = undef;
    $self->{'ReadHeading'}     = undef;
    $self->{'ReadTextBox'}     = undef;
    $self->{'ReadDate'}        = undef;
    $self->{'ReadList'}        = undef;
    $self->{'ReadComments'}    = undef;
    $self->{'ReadProgress'}    = undef;
    $self->{'ReadStatus'}      = undef;
    $self->{'WriteHeading'}    = undef;
    $self->{'WriteTextBox'}    = undef;
    $self->{'WriteComments'}   = undef;
    $self->{'WriteStatus'}     = undef;
    $self->{'WriteProgress'}   = undef;
    $self->{'WriteUpload'}     = undef;
    $self->{'PreviewHeading'}  = undef;
    $self->{'PreviewTextBox'}  = undef;
    $self->{'PreviewSheet'}    = undef;
    $self->{'OfflineTable'} = undef;
    $self->{'OfflineSheet'}    = undef;
    $self->{'TabView'}         = undef;
    
    # Attributes
    $self->{'ReadingEntries'}   = undef;
    $self->{'JournalHost'}      = 'use.perl.org'; # Should get this from somewhere...
    $self->{'ReadingListPlace'} = 0; # Last entry listed on reading list
    
    bless ($self, $class);

    $self->{'NSWindowController'} = NSWindowController->alloc->initWithWindowNibName_owner("MainWindow", $self);
    $self->{'NSWindowController'}->window;
    $self->{'NSWindowController'}->setWindowFrameAutosaveName("JournalWindow");

    # Set the window title
    $self->{'Window'}->setTitle($self->{'JournalHost'}." Journal");
    
    # Setup the progress indicators
    $self->{'ReadProgress'}->setUsesThreadedAnimation(1);
    $self->{'WriteProgress'}->setUsesThreadedAnimation(1);
    
    # Fill in the poplist for the reading tab
    $self->writeNewEntry;
    $self->populateReadList;
    
    # Offer to upload any offline entries
    $self->checkOfflineEntries;
    
    #$self->{'Window'}->setWindowFrameAutosaveName("JournalWindow");

    return $self;
}

sub readListPick {

  my $self = shift;
  my $journal = $self->{'journal'};
  my $popList = $self->{'ReadList'};

  # Check to see if they are asking for an entry or for us to get
  # a new list of entries
  if(($popList->indexOfSelectedItem == $popList->numberOfItems - 1) or
     ($popList->indexOfSelectedItem == 0 and $popList->titleOfSelectedItem eq "Get Previous Entries...")) {
    $self->populateReadList($popList->titleOfSelectedItem);
  }
  else {
    # Start the spinning pole
    $self->{'ReadProgress'}->startAnimation($self);

    # Get the id of the selected entry  
    my $id = $self->{'ReadingEntries'}->[$popList->indexOfSelectedItem];

    # Create journal object and get the entry from the server
    my $entry = JournalEntry->new('host' => $self->{'JournalHost'},
                                  'id'   => $id);
    $entry->retrieveEntry();

    ## Format the HTML elements
    my $bodyHTML = $self->createAttributedHTMLString($entry->formattedBody());
    my $commentHTML = $entry->comments() ? $self->createAttributedHTMLString('<a href="'.$entry->comments().'">Comments</a>') : NSMutableAttributedString->alloc->initWithString("");

    # Update the GUI elements
    $self->{'ReadHeading'}->setStringValue($entry->subject());  
    $self->{'ReadTextBox'}->textStorage->setAttributedString($bodyHTML);
    $self->{'ReadDate'}->setStringValue($entry->date());
    $self->{'ReadComments'}->textStorage->setAttributedString($commentHTML);

    # Stop the spinning pole
    $self->{'ReadProgress'}->stopAnimation($self);
  }
}

sub writeSave {

  my $self = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  
  # Clear any status message
  $self->clearStatusMessage($self->{'WriteStatus'});
  
  # Build up an object to store in the array
  my $entry = new JournalEntry( 'host' => $self->{'JournalHost'} );
  $entry->body($self->{'WriteTextBox'}->string);
  $entry->subject($self->{'WriteHeading'}->stringValue);
  $entry->comments($self->{'WriteComments'}->state);
  $entry->type($userDefaults->integerForKey("writePostType"));
  
  # Tell the entry to save itself
  if($entry->saveEntry) {
    $self->{'WriteStatus'}->setTextColor(NSColor->blackColor);
    $self->{'WriteStatus'}->setStringValue("Entry Saved");
  }
  else {
    $self->{'WriteStatus'}->setTextColor(NSColor->redColor);
    $self->{'WriteStatus'}->setStringValue("Error Saving Entry");
  }
}

sub writePreview {

  my $self = shift;
  my $sender = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  
  # Clear any status message
  $self->clearStatusMessage($self->{'WriteStatus'});

  my $entry = JournalEntry->new( 'host' => $self->{'JournalHost'} );

  # Set the details for the new journal entry
  $entry->body($self->{'WriteTextBox'}->string);
  $entry->subject($self->{'WriteHeading'}->stringValue);
  $entry->type($userDefaults->integerForKey("writePostType") + 1);

  # Format the HTML elements
  my $bodyHTML = $self->createAttributedHTMLString($entry->formattedBody());

  $self->{'PreviewHeading'}->setStringValue($entry->subject());  
  $self->{'PreviewTextBox'}->textStorage->setAttributedString($bodyHTML);

  # Load the preview sheet
NSApplication->sharedApplication->beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo($self->{'PreviewSheet'},
                                                                            $sender->window,
                                                                            $self,
                                                                            'sheetDidEnd:returnCode:contextInfo:',
                                                                            undef);

}

sub writeUpload {

  my $self = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;

  # Clear any status message
  $self->clearStatusMessage($self->{'WriteStatus'});

  my $entry = JournalEntry->new( 'host' => $self->{'JournalHost'} );

  # Set the details for the new journal entry
  $entry->body($self->{'WriteTextBox'}->string);
  $entry->subject($self->{'WriteHeading'}->stringValue);
  $entry->comments($self->{'WriteComments'}->state);
  $entry->type($userDefaults->integerForKey("writePostType") + 1);

  # Start the spinning pole
  $self->{'WriteProgress'}->startAnimation($self);

  # Upload the new entry to the server
  if($entry->uploadEntry()) {
    $self->{'WriteStatus'}->setTextColor(NSColor->blackColor);
    $self->{'WriteStatus'}->setStringValue("Entry Uploaded");
  }
  else {
    $self->{'WriteStatus'}->setTextColor(NSColor->redColor);
    $self->{'WriteStatus'}->setStringValue("Error Uploading Entry");
  }

  # Stop the spinning pole
  $self->{'WriteProgress'}->stopAnimation($self);
}

sub writeNewEntry {

  my $self = shift;
  my $sender = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;

  # Make sure we are on the writing tab
  $self->{'TabView'}->selectTabViewItemAtIndex(0);
  
  # Set all the GUI elements back to their defaults
  $self->{'WriteTextBox'}->setString("");
  $self->{'WriteHeading'}->setStringValue("");
  $self->{'WriteComments'}->setState($userDefaults->boolForKey("writeEnableComments"));
  $self->{'WriteComments'}->setEnabled(1);
  
  # Ensure the upload button is correct
  $self->{'WriteUpload'}->setAction('writeUpload:');
  $self->{'WriteUpload'}->setTitle('Upload');
  
  $self->clearStatusMessage($self->{'WriteStatus'});
}

sub writeOverwrite {

  my $self = shift;
  my $sender = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;

  # Clear any status message
  $self->clearStatusMessage($self->{'WriteStatus'});

  my $entry = JournalEntry->new( 'host' => $self->{'JournalHost'},
                                  'id'  => $self->{'EditEntry'} );

  # Set the details for the new journal entry
  $entry->body($self->{'WriteTextBox'}->string);
  $entry->subject($self->{'WriteHeading'}->stringValue);
  $entry->comments($self->{'WriteComments'}->state);
  $entry->type($userDefaults->integerForKey("writePostType") + 1);

  # Start the spinning pole
  $self->{'WriteProgress'}->startAnimation($self);

  # Upload the new entry to the server
  if($entry->overwriteEntry()) {
    $self->{'WriteStatus'}->setTextColor(NSColor->blackColor);
    $self->{'WriteStatus'}->setStringValue("Entry Overwriting Entry");
  }
  else {
    $self->{'WriteStatus'}->setTextColor(NSColor->redColor);
    $self->{'WriteStatus'}->setStringValue("Error Overwriting Entry");
  }

  # Stop the spinning pole
  $self->{'WriteProgress'}->stopAnimation($self);
}

sub editEntry {

  my $self = shift;
  my $sender = shift;
  my $popList = $self->{'ReadList'};
  
  # Clear any status message
  $self->clearStatusMessage($self->{'WriteStatus'});
  
  # Get the id of the selected entry  
  my $id = $self->{'ReadingEntries'}->[$popList->indexOfSelectedItem];
  
  # Set the write mode to edit
  $self->{'TabView'}->selectTabViewItemAtIndex(0);
  $self->{'WriteUpload'}->setAction('writeOverwrite:');
  $self->{'WriteUpload'}->setTitle('Overwrite');
  $self->{'EditEntry'} = $id;

  # Start the spinning pole
  $self->{'WriteProgress'}->startAnimation($self);

  my $entry = JournalEntry->new( 'host' => $self->{'JournalHost'},
                                  'id'   => $id );
  $entry->retrieveEntry();
  
  $self->{'WriteTextBox'}->setString($entry->body());
  $self->{'WriteHeading'}->setStringValue($entry->subject());
  if($entry->comments()) {
    $self->{'WriteComments'}->setState(1);
    $self->{'WriteComments'}->setEnabled(0); # Can't turn off comments
  }
  else {
    $self->{'WriteComments'}->setState(0);
  }
  
  # Stop the spinning pole
  $self->{'WriteProgress'}->stopAnimation($self);
}

sub deleteEntry {

  my $self = shift;
  my $sender = shift;
  my $popList = $self->{'ReadList'};
  
  # Get the id of the selected entry  
  my $id = $self->{'ReadingEntries'}->[$popList->indexOfSelectedItem];
  
  my $title   = "Delete";
  my $message = "Are you sure you want to delete this entry?";
  my $default = "Yes";
  my $alter   = "No";
  my $empty   = "";
  
  my $result = NSRunAlertPanel(\$title, \$message, \$default, \$alter, \$empty);
  
  if($result == 1) {
    my $entry = JournalEntry->new( 'host' => $self->{'JournalHost'},
                                   'id'   => $id );

    # Start the spinning pole
    $self->{'ReadProgress'}->startAnimation($self);

    if($entry->deleteEntry) {
      $self->{'ReadStatus'}->setTextColor(NSColor->blackColor);
      $self->{'ReadStatus'}->setStringValue("Entry successfully deleted");
    }
    else {
      $self->{'ReadStatus'}->setTextColor(NSColor->redColor);
      $self->{'ReadStatus'}->setStringValue("Error deleting entry!");
    }

    # Stop the spinning pole
    $self->{'ReadProgress'}->stopAnimation($self);
    $self->populateReadList("Reload");
  }
}

sub dismissPreview {

  my $self = shift;
  my $sender = shift;
  
  $self->{'PreviewSheet'}->orderOut($sender);
  
  NSApplication->sharedApplication->endSheet_returnCode($self->{'PreviewSheet'}, 1);
}

sub offlineUpload {

  my $self = shift;
  my $sender = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  my $offlineEntries = $userDefaults->objectForKey("writeSavedEntries");
  my $mutArray = NSMutableArray->arrayWithArray($offlineEntries);
  my $uploadCount = 0;

  # Clear any status message
  $self->clearStatusMessage($self->{'WriteStatus'});
  
  # Start the spinning poles, both as we aren't certain which
  # tab will be visable
  $self->{'WriteProgress'}->startAnimation($self);
  $self->{'ReadProgress'}->startAnimation($self);

  # Determine which entries we should upload, reverse
  # order so we can remove from the array without disturbing
  # the order
  for($i = scalar(@{ $self->{'OfflineSelection'} }) - 1;$i >= 0;$i--) {
    if($self->{'OfflineSelection'}->[$i]) {
      $offlineEntry = $mutArray->objectAtIndex($i);
    
      # Get a new journal entry, and populate
      my $entry = JournalEntry->new( 'host' => $self->{'JournalHost'} );
      $entry->body($offlineEntry->objectForKey("entry"));
      $entry->subject($offlineEntry->objectForKey("subject"));
      $entry->comments($offlineEntry->objectForKey("comments")->intValue);
      $entry->type($offlineEntry->objectForKey("posttype")->intValue + 1);

      # Wait ten seconds between posts
      sleep(10) if $uploadCount;
      # Upload the entry, and if successful, remove from array
      if($entry->uploadEntry()) {
        $mutArray->removeObjectAtIndex($i);
        $uploadCount++;
      }
      else {
        $self->{'WriteStatus'}->setTextColor(NSColor->redColor);
        $self->{'WriteStatus'}->setStringValue("Error Uploading Offline Entry: ".$uploadCount." entries uploaded successfully");
        break;
      }
    }
  }

  #$self->{'WriteStatus'}->setTextColor(NSColor->blackColor);
  #$self->{'WriteStatus'}->setStringValue($uploadCount." entries uploaded successfully");
  
  # Save the offline array
  $userDefaults->removeObjectForKey("writeSavedEntries");
  $userDefaults->setObject_forKey($mutArray, "writeSavedEntries");
  $self->{'OfflineSelection'} = $mutArray->count();
  
  # Refresh the list of entries
  $self->{'OfflineTable'}->reloadData();

  # Stop the spinning poles
  $self->{'WriteProgress'}->stopAnimation($self);
  $self->{'ReadProgress'}->stopAnimation($self);

}

sub offlineDelete {

  my $self = shift;
  my $sender = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  my $offlineEntries = $userDefaults->objectForKey("writeSavedEntries");
  my $mutArray = NSMutableArray->arrayWithArray($offlineEntries);
  
  # Figure out which entries we should remove, reverse
  # order so we can remove from the array without disturbing
  # the order
  for($i = scalar(@{ $self->{'OfflineSelection'} }) - 1;$i >= 0;$i--) {
    if($self->{'OfflineSelection'}->[$i]) {
      $mutArray->removeObjectAtIndex($i);
    }
  }

  # Save the offline array
  $userDefaults->removeObjectForKey("writeSavedEntries");
  $userDefaults->setObject_forKey($mutArray, "writeSavedEntries");
  $userDefaults->synchronize();
  $self->{'OfflineSelection'} = $mutArray->count();
  
  # Refresh the list of entries
  $self->{'OfflineTable'}->reloadData();
}

sub offlineDismiss {

  my $self = shift;
  my $sender = shift;

  # Clear any status message
  $self->clearStatusMessage($self->{'WriteStatus'});

  # Get rid of the sheet
  $self->{'OfflineSheet'}->orderOut($sender);
  NSApplication->sharedApplication->endSheet_returnCode($self->{'OfflineSheet'}, 1);
}

sub sheetDidEnd_returnCode_contextInfo {
  my $self = shift;

  # Nothing to do...  
}

sub numberOfRowsInTableView {

  my $self = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  my $offlineEntries = $userDefaults->objectForKey("writeSavedEntries");

  return $offlineEntries->count;
}

sub tableView_objectValueForTableColumn_row {

  my $self = shift;
  my ($table, $column, $row) = @_;

  my $userDefaults = NSUserDefaults->standardUserDefaults;
  my $offlineEntries = $userDefaults->objectForKey("writeSavedEntries");
  my $ident = $column->identifier();
  my $offlineEntry = $offlineEntries->objectAtIndex($row);

  if($ident eq "subject") {
    my $button = NSButtonCell->new;
    $column->setDataCell($button);
    $button->setButtonType(3); # Should be a switch
    $button->setTitle($offlineEntry->objectForKey($ident));
    $button->setState($self->{'OfflineSelection'}->[$row]);
    $button->setControlSize(1); # Make it small
    $button->setTarget($self);
    $button->setAction('clickOfflineEntry:');
    return $button;
  }
  elsif($ident eq "creation_date") {
    $date = NSCalendarDate->dateWithString_calendarFormat($offlineEntry->objectForKey($ident),
                                                          "%a %b %e %H:%M:%S %Y");
    return $date;
  }
}

sub clickOfflineEntry {

  my $self = shift;
  my $sender = shift;

  $self->{'OfflineSelection'}->[$sender->clickedRow()] = !$self->{'OfflineSelection'}->[$sender->clickedRow()];
}

sub refreshReadingList {
  my $self = shift;
  
  $self->populateReadList();
}

sub populateReadList {

  my $self = shift;
  my $type = shift; # Are we getting more or less
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  my $poplist = $self->{'ReadList'};
  
  my $host = $self->{'JournalHost'};
  my $uid = $userDefaults->stringForKey($host."_UserID");
  my $password = $userDefaults->stringForKey($host."_Password");
  
  # Figure out whether how many entries we need to retrive to get the ones
  # we want to list out
  my $entriesToGet = $self->{'ReadingListPlace'};
  if($type eq "Get Previous Entries...") {
    $entriesToGet -= $userDefaults->integerForKey("readingListCount");
    if($entriesToGet < $userDefaults->integerForKey("readingListCount")) {
      $entriesToGet = $userDefaults->integerForKey("readingListCount");
    }
  }
  elsif($type eq "Reload") {
    $entriesToGet = $self->{'ReadingListPlace'};
  }
  else {
    $entriesToGet += $userDefaults->integerForKey("readingListCount");
  }

  $poplist->removeAllItems;

  # Only try to get the entries if we know the uid
  if($uid) {
    my $recent_entries;
    my $uri = "http://$host/Slash/Journal/SOAP"; 
    my $proxy = "http://$host/journal.pl";
    my $cookie = HTTP::Cookies->new;
    $cookie->set_cookie(0, user => $self->bakeUserCookie($uid, $password), '/', $host);
    my $journal = SOAP::Lite->uri($uri)
                            ->proxy($proxy, cookie_jar => $cookie);

    # Start the spinning poles
    $self->{'ReadProgress'}->startAnimation($self);
    $self->{'WriteProgress'}->startAnimation($self);
  
    eval { $recent_entries = $journal->get_entries($uid, $entriesToGet)->result; };

    # If we got something
    if($recent_entries > 0) {
      undef $self->{'ReadingEntries'};

      # If there are previos entries, add a title
      if($entriesToGet  - $userDefaults->integerForKey("readingListCount") > 0) {
        $poplist->addItemWithTitle("Get Previous Entries...");
        push @{ $self->{'ReadingEntries'} }, "garbage";
      }

      if($userDefaults->integerForKey("readingListCount") < scalar @{ $recent_entries }) {
        $i = $userDefaults->integerForKey("readingListCount");
      }
      else {
        $i = scalar @{ $recent_entries };
      }
      
      while ($i > 0) {
        $entry = @{ $recent_entries }[-$i];
        $poplist->addItemWithTitle($entry->{subject});
        push @{ $self->{'ReadingEntries'} }, $entry->{id};
        $i--;
      }
      
      if($entriesToGet <= scalar @{ $recent_entries } ) {
        # Add an entry to get extra titles
        $poplist->addItemWithTitle("Get More Entries...");
        $self->{'ReadingListPlace'} = $entriesToGet;
      }
      else {
        $self->{'ReadingListPlace'} = scalar @{ $recent_entries };
      }
      $self->{'ReadingListPlace'} - $userDefaults->integerForKey("readingListCount") > 0 ? $poplist->selectItemAtIndex(1) : $poplist->selectItemAtIndex(0);
      
      # Get the details of the selected item
      $self->readListPick($self);
    }
    # Stop the spinning poles
    $self->{'ReadProgress'}->stopAnimation($self);
    $self->{'WriteProgress'}->stopAnimation($self);

  }
}

sub checkOfflineEntries {

  my $self = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  my $offlineEntries = $userDefaults->objectForKey("writeSavedEntries");

  # If there are any offline entries, offer to upload them
  if($offlineEntries->count > 0) {
    $self->displayOfflineEntries();
  }
}

sub displayOfflineEntries {
  my $self = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  my $offlineEntries = $userDefaults->objectForKey("writeSavedEntries");

  # Everything is selected to start
  @temp = (1) x $offlineEntries->count;
  $self->{'OfflineSelection'} = \@temp;
 NSApplication->sharedApplication->beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo($self->{'OfflineSheet'},
                                                                            $self->{'Window'},
                                                                            $self,
                                                                            'sheetDidEnd:returnCode:contextInfo:',
                                                                            undef);

}

sub createAttributedHTMLString {

  my $self = shift;
  my $htmlString = shift;
  my $file_location = "/tmp/journalX.$$";
  my $data, $attributedString;
  my %emptyHash = {};

  # Write the html to a temp file
  open(FH, ">".$file_location);
  print FH $htmlString;
  close FH;

  # Create the attributedString object
  $data = NSData->dataWithContentsOfFile($file_location);
  $attributedString = NSMutableAttributedString->alloc->initWithHTML_documentAttributes($data, NSDictionary->dictionary);

  # Remove the temp file
  unlink $file_location;

  return $attributedString;
}

sub clearStatusMessage {

  my $self = shift;
  my $status = shift;

  # Clear any status message
  $status->setTextColor(NSColor->blackColor);
  $status->setStringValue("");
  $self->{'Window'}->display;

}


sub bakeUserCookie {
  my($self, $uid, $passwd) = @_;
  my $cookie = $uid . '::' . md5_hex($passwd);
  $cookie =~ s/(.)/sprintf("%%%02x", ord($1))/ge;
  $cookie =~ s/%/%25/g;
  return $cookie;
}

1;

#
# JournalController.pm
# Copyright 2002 Garth Douglass
# This program is distributed under the terms of the GNU General Public License, see COPYING file for details
#
