package JournalPreference;

use Foundation;
use Foundation::Functions;
use AppKit;
use AppKit::Functions;

@ISA = qw(Exporter);

sub new {
    # Typical Perl constructor
    # See 'perltoot' for details
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {
        'wc' => undef
    };

    # Outlets
    $self->{'ReadingListCount'}    = undef;
    $self->{'SiteList'}            = undef;
    $self->{'SitePassword'}        = undef;
    $self->{'SiteUserID'}          = undef;
    $self->{'SiteUserName'}        = undef;
    $self->{'Window'}              = undef;
    $self->{'WriteEnableComments'} = undef;
    $self->{'WritePostType'}       = undef;

    bless ($self, $class);

    # Load the nib file
    $self->{'NSPreferenceController'} = NSWindowController->alloc->initWithWindowNibName_owner("Preferences", $self);
    $self->{'NSPreferenceController'}->window;
    
    # Set the prefernce window with the user defaults
    my $userDefaults = NSUserDefaults->standardUserDefaults;
    $self->{'ReadingListCount'}->setIntValue($userDefaults->integerForKey("readingListCount"));
    $self->{'WriteEnableComments'}->setState($userDefaults->boolForKey("writeEnableComments"));
    $self->{'WritePostType'}->selectCellAtRow_column($userDefaults->integerForKey("writePostType"), 0);
    
    $siteList = $userDefaults->objectForKey("siteList");
    $self->{'SiteList'}->removeAllItems();
    for(my $i = 0; $i < $siteList->count();$i++) {
      $self->{'SiteList'}->addItemWithTitle($siteList->objectAtIndex($i));
    }
    
    # Populate the site details
    $self->populateSiteDetails();
    
    $self->{'Window'}->makeKeyAndOrderFront($self);
    

    return $self;
}

sub windowDidLoad {
    NSLog("window did load?");
}

sub awake {
    my $self = shift;
        
    $self->{'Window'}->makeKeyAndOrderFront($self);
}

sub populateSiteDetails {
    my $self = shift;
    my $site = $self->{'SiteList'}->titleOfSelectedItem();
    my $userDefaults = NSUserDefaults->standardUserDefaults;

    # Update the GUI elements with the details for the selected site    
    $self->{'SiteUserName'}->setStringValue($userDefaults->stringForKey($site."_UserName") || "");
    $self->{'SitePassword'}->setStringValue($userDefaults->stringForKey($site."_Password") || "");
    $self->{'SiteUserID'}->setStringValue($userDefaults->stringForKey($site."_UserID") || "");
}

sub savePreferences {

    my $self = shift;
    my $sender = shift;
    my $userDefaults = NSUserDefaults->standardUserDefaults;
    
    # Save reading preferences
    $userDefaults->setInteger_forKey($self->{'ReadingListCount'}->intValue, "readingListCount");
    
    # Save writing preferences
    $userDefaults->setBool_forKey($self->{'WriteEnableComments'}->state, "writeEnableComments");
    $userDefaults->setInteger_forKey($self->{'WritePostType'}->selectedRow, "writePostType");
    
    # Save site details
    my $site = $self->{'SiteList'}->titleOfSelectedItem();
    $userDefaults->setObject_forKey($self->{'SiteUserName'}->stringValue, $site."_UserName");
    $userDefaults->setObject_forKey($self->{'SiteUserID'}->stringValue, $site."_UserID");
    
    $userDefaults->setObject_forKey($self->{'SitePassword'}->stringValue, $site."_Password");

    $sender->window()->close();
}

sub discardPreferences {

    my $self = shift;
    my $sender = shift;
    
    $sender->window()->close();
}

#
# Stolen from example by pudge at http://use.perl.org/~pudge/journal/3459
# 
sub bakeUserCookie {
  my($self, $uid, $passwd) = @_;
  my $cookie = $uid . '::' . md5_hex($passwd);
  $cookie =~ s/(.)/sprintf("%%%02x", ord($uid))/ge;
  $cookie =~ s/%/%25/g;
  return $cookie;
}

1;

#
# JournalPreference.pm
# Copyright 2002 Garth Douglass
# This program is distributed under the terms of the GNU General Public License, see COPYING file for details
#

