package JournalX;

use Foundation;
use Foundation::Functions;
use AppKit;
use AppKit::Functions;

use JournalController;
use JournalPreference;

@ISA = qw(Exporter);

sub new {
    # Typical Perl constructor
    # See 'perltoot' for details
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {
        'wc' => undef,
        'NSPreferenceWindow' => undef
    };
    bless ($self, $class);

    return $self;
}

sub applicationWillFinishLaunching {
    my ($self, $notification) = @_;
    
    # Get a handle on the standard defaults object
    my $userDefaults = NSUserDefaults->standardUserDefaults;

    # Register the default preferences.
    my $registrationDefaults = NSMutableDictionary->alloc->init;
    my $siteList = NSMutableArray->alloc->init;
    $siteList->addObject("use.perl.org");
    $registrationDefaults->setObject_forKey($siteList, "siteList");
    $registrationDefaults->setObject_forKey(10, "readingListCount");
    $registrationDefaults->setObject_forKey(1, "writeEnableComments"); # Enabled
    $registrationDefaults->setObject_forKey(0, "writePostType"); # Plain text
    $registrationDefaults->setObject_forKey(NSMutableArray->array, "writeSavedEntries"); # an array for offline entries
    $userDefaults->registerDefaults($registrationDefaults);


    # Create the new controller object
    $self->{'wc'} = new JournalController;
    #$self->{'wc'}->setWindowFrameAutosaveName("JournalWindow");
    
    return 1;
}

sub showPreferences {
    my $self = shift;

    # Only create it if needed
    $self->{'NSPreferenceWindow'} = new JournalPreference unless $self->{'NSPreferenceWindow'};
    $self->{'NSPreferenceWindow'}->awake;

    return 1;
}

sub showOfflineEntries {
    my $self = shift;

    $self->{'wc'}->displayOfflineEntries();
}

sub showNewEntry {
  my $self = shift;

  $self->{'wc'}->writeNewEntry();
}


sub refreshReadingList {
    my $self = shift;
    
    $self->{'wc'}->refreshReadingList;
}

1;

#
# JournalX.pm
# Copyright 2002 Garth Douglass
# This program is distributed under the terms of the GNU General Public License, see COPYING file for details
#

